"use strict";
exports.__esModule = true;
exports.Handlers = exports.track = exports.usage = exports.onError = exports.addLogTelemetry = exports.removeMetadata = exports.addMetadata = exports.configure = exports.uninstall = exports.install = exports.isInstalled = void 0;
var types_1 = require("./types");
var Agent_1 = require("./Agent");
var express_1 = require("./handlers/express");
var telemetry_1 = require("./telemetry");
var AgentRegistrar_1 = require("./AgentRegistrar");
var watchers_1 = require("./watchers");
var isType_1 = require("./utils/isType");
var serialize_1 = require("./utils/serialize");
var Fault_1 = require("./Fault");
var TrackJSCapturePayload_1 = require("./types/TrackJSCapturePayload");
var watchers = [watchers_1.ConsoleWatcher, watchers_1.ExceptionWatcher, watchers_1.NetworkWatcher, watchers_1.RejectionWatcher];
var hasInstalled = false;
function isInstalled() {
    return hasInstalled;
}
exports.isInstalled = isInstalled;
function install(options) {
    if (hasInstalled) {
        throw new types_1.TrackJSError("already installed.");
    }
    if (!options) {
        throw new types_1.TrackJSError("install options are required.");
    }
    if (!options.token) {
        throw new types_1.TrackJSError("install token is required.");
    }
    try {
        AgentRegistrar_1.AgentRegistrar.init(new Agent_1.Agent(options));
        watchers.forEach(function (w) { return w.install(); });
        AgentRegistrar_1.AgentRegistrar.getCurrentAgent().captureUsage();
        hasInstalled = true;
    }
    catch (error) {
        (0, Fault_1.captureFault)(error);
        throw new types_1.TrackJSError("error occurred during installation.", error);
    }
}
exports.install = install;
function uninstall() {
    if (!hasInstalled) {
        return;
    }
    try {
        watchers.forEach(function (w) { return w.uninstall(); });
        AgentRegistrar_1.AgentRegistrar.close();
        hasInstalled = false;
    }
    catch (error) {
        (0, Fault_1.captureFault)(error);
        throw new types_1.TrackJSError("error occurred during uninstall.", error);
    }
}
exports.uninstall = uninstall;
function configure(options) {
    if (!hasInstalled) {
        throw new types_1.TrackJSError("not installed.");
    }
    AgentRegistrar_1.AgentRegistrar.getCurrentAgent().configure(options);
}
exports.configure = configure;
function addMetadata(meta, value) {
    if (!hasInstalled) {
        throw new types_1.TrackJSError("not installed.");
    }
    AgentRegistrar_1.AgentRegistrar.getCurrentAgent().metadata.add(meta, value);
}
exports.addMetadata = addMetadata;
function removeMetadata(meta) {
    if (!hasInstalled) {
        throw new types_1.TrackJSError("not installed.");
    }
    AgentRegistrar_1.AgentRegistrar.getCurrentAgent().metadata.remove(meta);
}
exports.removeMetadata = removeMetadata;
function addLogTelemetry(severity) {
    var messages = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        messages[_i - 1] = arguments[_i];
    }
    if (!hasInstalled) {
        throw new types_1.TrackJSError("not installed.");
    }
    AgentRegistrar_1.AgentRegistrar.getCurrentAgent().telemetry.add("c", new telemetry_1.ConsoleTelemetry(severity, messages));
}
exports.addLogTelemetry = addLogTelemetry;
function onError(func) {
    if (!hasInstalled) {
        throw new types_1.TrackJSError("not installed.");
    }
    AgentRegistrar_1.AgentRegistrar.getCurrentAgent().onError(func);
}
exports.onError = onError;
function usage() {
    if (!hasInstalled) {
        throw new types_1.TrackJSError("not installed.");
    }
    AgentRegistrar_1.AgentRegistrar.getCurrentAgent().captureUsage();
}
exports.usage = usage;
function track(data, options) {
    if (!hasInstalled) {
        throw new types_1.TrackJSError("not installed.");
    }
    var error = (0, isType_1.isError)(data) ? data : new Error((0, serialize_1.serialize)(data));
    if (options) {
        AgentRegistrar_1.AgentRegistrar.getCurrentAgent()
            .clone(options)
            .captureError(error, TrackJSCapturePayload_1.TrackJSEntry.Direct);
    }
    else {
        AgentRegistrar_1.AgentRegistrar.getCurrentAgent().captureError(error, TrackJSCapturePayload_1.TrackJSEntry.Direct);
    }
    return error;
}
exports.track = track;
exports.Handlers = {
    expressErrorHandler: function () {
        if (!hasInstalled) {
            throw new types_1.TrackJSError("not installed.");
        }
        return (0, express_1.expressErrorHandler)();
    },
    expressRequestHandler: function () {
        if (!hasInstalled) {
            throw new types_1.TrackJSError("not installed.");
        }
        return (0, express_1.expressRequestHandler)();
    }
};
